// The contents of this file are used with permission, subject to
// the Mozilla Public License Version 1.1 (the "License"); you may
// not use this file except in compliance with the License. You may
// obtain a copy of the License at
// http://www.mozilla.org/MPL/MPL-1.1.html
//
// Software distributed under the License is distributed on an
// "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
{$I DnConfig.inc}
unit DnTcpReactor;

interface
uses
  Windows, Winsock2, SysUtils, Classes, Contnrs,
  DnRtl, DnConst, DnInterfaces, DnSimpleExecutor,
  DnAbstractExecutor, DnAbstractLogger, DnAbstractTimer,
  DnTimer, DnTimeOutSupport;

type
  TDnTcpChannel = class;
  TDnTcpReactor = class;

  TDnReqContext = record
    FOverlapped:  TOverlapped;
    FRequest:     Pointer;
    FReqRouting:  Boolean;
  end;

  PDnReqContext = ^TDnReqContext;

  TDnTcpRequest = class(TDnObject, IDnIORequest, IDnIOResponse)
  protected
    FChannel:     IDnChannel;
    FContext:     TDnReqContext;
    FKey:         Pointer;
    FStartBuffer: Pointer;
    FTotalSize:   Cardinal;
    FErrorCode:   Cardinal;
    FRTContext:   TDnThreadContext;

    procedure PostError(ErrorCode: Integer; WasRead: Cardinal);
    procedure CatchError;

    //IDnIORequest
    procedure Execute; virtual;
    function  IsComplete: Boolean; virtual;
    procedure ReExecute; virtual; abstract;
    function  RequestType: TDnIORequestType; virtual; abstract;
    //IDnIOResponse
    function  Channel: IDnIOTrackerHolder;
    procedure CallHandler(Context: TDnThreadContext); virtual; abstract;

    procedure SetTransferred(Transferred: Cardinal); virtual; abstract;
    function  IsCPUNeeded: Boolean; virtual; abstract;
    procedure Cancel;
  public
    constructor Create(Channel: IDnChannel; Key: Pointer);
    destructor Destroy; override;
  end;


  TDnTcpChannel = class (TDnObject, IDnIOTrackerHolder, IDnImplementation,
                          IUnknown, IDnChannel)
  protected
    FRemoteAddr:          TSockAddrIn;
    FSocket:              TSocket;
    FReactor:             TDnTcpReactor;

    FReadQueue:           TInterfaceList;
    FWriteQueue:          TInterfaceList;
    FClosingRequest:      IDnIORequest;
    FConnectingRequest:   IDnIORequest;
    FRunGuard:            TDnMutex;

    FTracker:             Pointer;
    FCache:               String;
    FCustomData:          Pointer;
    FFinishedIOCount:     Cardinal;
    FTimeOut:             Cardinal;
    FTimeOutObserver:     IDnTimerObserver;
    FTimeOutGuard:        TDnMutex;
    FClosing:             Boolean;
    FTimeOutExpired:      Boolean;
    FTimeOutAbort:        Boolean;
    FFirstTimeOutRequest: Boolean;
    
    class function    MatchingRequest(Context: Pointer): TDnTcpRequest;
    procedure   ExecuteNext(Request: IDnIORequest);
    procedure   SetTimeOut(Value: Cardinal);
    procedure   IssueTimeOutRequest(TimeOut: Cardinal; Request: IDnIORequest);
    procedure   InitChannel;
    function    GetRemoteAddrPtr: Pointer;
    procedure   IncrementIOCounter;
  public
    class function CheckImpl(obj: IUnknown): TDnTcpChannel;
    constructor Create(Reactor: TDnTcpReactor; Sock: TSocket; RemoteAddr: TSockAddrIn);
    constructor CreateEmpty(Reactor: TDnTcpReactor; const RemoteIP: String; Port: Word);
    destructor  Destroy; override;

    procedure   CloseSocketHandle;
    procedure   StopTimeOutTracking;

    function    CacheSize: Cardinal;
    function    CachePtr: PChar;
    function    Add2Cache(Block: PChar; BlockSize: Cardinal): Cardinal;
    function    ExtractFromCache(Block: PChar; BlockSize: Cardinal): Cardinal;
    procedure   InsertToCache(Block: PChar; BlockSize: Cardinal);
    function    CacheHasData: Boolean;
    function    IsClosed: Boolean;

    //IDnChannel
    function  RemotePort: Word;
    function  RemoteAddr: String;
    function  RemoteHost: String;
    procedure SetCustomData(P: Pointer);
    function  GetCustomData: Pointer;
    function  IsClosing: Boolean;
    //IDnIOTrackerHolder
    function  IsBound: Boolean;
    procedure Bind(Tracker: Pointer);
    procedure Unbind(Tracker: Pointer);
    function  Tracker: Pointer;

    procedure RunRequest(Request: IDnIORequest);
    procedure SetNagle(Value: Boolean);

    //IDnImplementation
    function GetImplementation: Pointer;

    property SocketHandle: TSocket read FSocket;
    property Reactor: TDnTcpReactor read FReactor;
    property TimeOut: Cardinal read FTimeOut write SetTimeOut;
    property RemoteAddrPtr: Pointer read GetRemoteAddrPtr;
    property TimeOutExpired: Boolean read FTimeOutExpired;
  end;

  TDnTcpReactorThread = class;
  TDnTimeOutExpiredEvent = procedure (Context: TDnThreadContext; Channel: IDnChannel) of object;
  
  {$IFDEF ROOTISCOMPONENT}
  TDnTcpReactor = class (TComponent, IDnTimeOutHandler)
  {$ELSE}
  TDnTcpReactor = class (TDnObject, IDnTimeOutHandler)
  {$ENDIF}
  protected
    FPort:                  THandle;

    FActive:                Boolean;
    FThread:                TDnTcpReactorThread;
    FChannels:              TInterfaceList;
    FGuard:                 TDnMutex;

    FLogger:                TDnAbstractLogger;
    FLogLevel:              TDnLogLevel;
    FExecutor:              TDnAbstractExecutor;

    FTimer:                 TDnAbstractTimer;
    FTimerExecutor:         TDnSimpleExecutor;
    FTimeOutExpiredEvent:   TDnTimeOutExpiredEvent;

    procedure SetActive(Value: Boolean);
    function  TurnOn: Boolean; virtual;
    function  TurnOff: Boolean; virtual;
    procedure TimeOutFired(Context: TDnThreadContext; Channel: IDnChannel;
                              ExpiredTacts: Cardinal; Key: Pointer);
    procedure OnTimeOutExpired(Context: TDnThreadContext; Channel: IDnChannel);
    function  GetChannelCount: Integer;
    {$IFDEF ROOTISCOMPONENT}
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    {$ENDIF}
  public
    constructor Create {$IFDEF ROOTISCOMPONENT}(AOwner: TComponent);override{$ENDIF}; 
    destructor  Destroy; override;
    procedure   PostChannel(Channel: IDnChannel);
    procedure   PostChannelError(Channel: TDnTcpChannel; Request: TDnTcpRequest);
    procedure   RemoveChannel(Channel: IDnChannel);
    function    MakeChannel(const IPAddress: String; Port: Word): IDnChannel;
    procedure   SetTimeout(Channel: IDnChannel; Value: Cardinal);
    procedure   CloseChannels;
  published
    property Active: Boolean read FActive write SetActive;
    property Executor: TDnAbstractExecutor read FExecutor write FExecutor;
    property Logger: TDnAbstractLogger read FLogger write FLogger;
    property LogLevel: TDnLogLevel read FLogLevel write FLogLevel;
    property PortHandle: THandle read FPort;
    property ChannelCount: Integer read GetChannelCount;
    property OnTimeOut: TDnTimeOutExpiredEvent read FTimeOutExpiredEvent write FTimeOutExpiredEvent;
  end;

  TDnTcpReactorThread = class(TDnThread)
  protected
    FReactor: TDnTcpReactor;
    procedure CreateContext; override;
    procedure DestroyContext; override;
    procedure ThreadRoutine; override;
    procedure DoRequest(Request: TDnTcpRequest; Channel: TDnTcpChannel);
    procedure DoClose(Channel: TDnTcpChannel; Request: TDnTcpRequest);
    procedure UpdateCounters(Channel: TDnTcpChannel; Request: TDnTcpRequest);
    procedure ParseIONotification(Transferred: Cardinal; Key: Cardinal; Overlapped: POverlapped);
    procedure ParseIOError(Transferred: Cardinal; Key: Cardinal; Overlapped: POverlapped);
    procedure LogMessage(S: String);
  public
    constructor Create(Reactor: TDnTcpReactor);
    destructor Destroy; override;
  end;


  procedure Register;

implementation
var ParseIONotificationCount: Integer;

constructor TDnTcpRequest.Create(Channel: IDnChannel; Key: Pointer);
begin
  inherited Create;
  FChannel := Channel;
  FTotalSize := 0;
  FStartBuffer := Nil;
  FRefCount := 0;
  FKey := Key;
  FErrorCode := 0;
  FRTContext := Nil;
end;

destructor TDnTcpRequest.Destroy;
begin
  inherited Destroy;
end;

procedure TDnTcpRequest.Cancel;
begin
  Self.FContext.FRequest := Nil;
end;

procedure TDnTcpRequest.PostError(ErrorCode: Integer; WasRead: Cardinal);
var ChannelImpl: TDnTcpChannel;
begin
  FErrorCode := ErrorCode;
  ChannelImpl := TDnTcpChannel.CheckImpl(FChannel);
  if LongBool(PostQueuedCompletionStatus( ChannelImpl.Reactor.PortHandle, WasRead,
                                Cardinal(Pointer(FChannel)), @FContext )) = False
  then
    raise EDnException.Create(ErrWin32Error, GetLastError(), 'PostQueuedCompletionStatus');
end;

procedure TDnTcpRequest.Execute;
var ChannelImpl: TDnTcpChannel;
begin
  FRTContext := GetCurrentContext;
  if FRTContext <> Nil then
    FRTContext.Grab;

  FErrorCode := 0;
  FillChar(FContext.FOverlapped, SizeOf(FContext.FOverlapped), 0);
  FContext.FRequest := Self;
  FContext.FReqRouting := True;

  ChannelImpl := TDnTcpChannel.CheckImpl(FChannel);
  if not ChannelImpl.FFirstTimeOutRequest and (ChannelImpl.FTimeOut <> 0) then
  begin
    ChannelImpl.FFirstTimeOutRequest := True;
    ChannelImpl.IssueTimeOutRequest(ChannelImpl.FTimeOut, Self);
  end;
end;

function  TDnTcpRequest.IsComplete: Boolean;
begin
  if FRTContext <> Nil then
    FRTContext.Release;
  Result := False;
end;


procedure TDnTcpRequest.CatchError;
begin
  FErrorCode := GetLastError;
end;

function  TDnTcpRequest.Channel: IDnIOTrackerHolder;
begin
  if FChannel<>Nil then
  begin
    Result := IDnIOTrackerHolder(FChannel);//as IDnIOTrackerHolder;
  end;
end;
//------------------------------------------------------------------------------

class function TDnTcpChannel.CheckImpl(obj: IUnknown): TDnTcpChannel;
var impl: IDnImplementation;
begin
  Result := Nil;
  if IDnChannel(obj) <> Nil then
  begin
    impl := obj as IDnImplementation;
    if impl <> Nil then
      Result := TDnTcpChannel(impl.GetImplementation())
  end;
end;


procedure TDnTcpChannel.InitChannel;
begin
  FTracker := Nil;
  FRefCount := 0;
  //FRequest := TInterfaceList.Create;
  FReadQueue := TInterfaceList.Create;
  FWriteQueue := TInterfaceList.Create;
  FClosingRequest := Nil;
  FConnectingRequest := Nil;
  FRunGuard := TDnMutex.Create;
  FCustomData := Nil;
  SetLength(FCache, 0);
  FTimeOut := 0;
  FTimeOutGuard := TDnMutex.Create;
  FClosing := False;
  FFinishedIOCount := 0;
  FTimeOutExpired := False;
  FTimeOutAbort := False;
  FFirstTimeOutRequest := False;
end;

constructor TDnTcpChannel.Create(Reactor: TDnTcpReactor; Sock: TSocket; RemoteAddr: TSockAddrIn);
begin
  inherited Create;
  FReactor := Reactor;
  FSocket := Sock;
  FRemoteAddr := RemoteAddr;
  InitChannel;
end;

//{$O-}
constructor TDnTcpChannel.CreateEmpty(Reactor: TDnTcpReactor; const RemoteIP: String; Port: Word);
begin
  inherited Create;
  FReactor := Reactor;
  FSocket := WSASocket(AF_INET, SOCK_STREAM, IPPROTO_TCP, Nil, 0, WSA_FLAG_OVERLAPPED);
  if FSocket = INVALID_SOCKET then
    raise EDnException.Create(ErrWin32Error, WSAGetLastError(), 'WSASocket');
  FillChar(FRemoteAddr, 0, SizeOf(FRemoteAddr));
  FRemoteAddr.sin_addr.S_addr := inet_addr(PChar(RemoteIP));
  FRemoteAddr.sin_port := htons(Port);
  FRemoteAddr.sin_family := AF_INET;
  InitChannel;
end;


function TDnTcpChannel.GetImplementation: Pointer;
begin
  Result := Pointer(TDnTcpChannel(Self));
end;
//{$O+}

function TDnTcpChannel.GetRemoteAddrPtr: Pointer;
begin
  Result := @FRemoteAddr;
end;

procedure TDnTcpChannel.IncrementIOCounter;
begin
  FTimeOutGuard.Acquire;
  Inc(FFinishedIOCount);
  FTimeOutGuard.Release;
end;

procedure TDnTcpChannel.IssueTimeOutRequest(TimeOut: Cardinal; Request: IDnIORequest);
begin
  FTimeOutGuard.Acquire;
  FTimeOutObserver := Nil;
  FTimeOutObserver := FReactor.FTimer.RequestTimerNotify(Self, TimeOut, Pointer(FFinishedIOCount));
  FTimeOutGuard.Release;
end;


procedure TDnTcpChannel.SetCustomData(P: Pointer);
begin
  FCustomData := P;
end;

function TDnTcpChannel.GetCustomData: Pointer;
begin
  Result := FCustomData;
end;

procedure TDnTcpChannel.SetTimeOut(Value: Cardinal);
begin
  if FTimeOut = 0 then
    FTimeOut := Value
  else
    raise EDnException.Create(ErrCannotSetTimeOutTwice, 0);
end;

function TDnTcpChannel.CacheSize: Cardinal;
begin
  Result := Length(FCache);
end;

function TDnTcpChannel.CachePtr: PChar;
begin
  Result := PChar(FCache);
end;

function TDnTcpChannel.Add2Cache(Block: PChar; BlockSize: Cardinal): Cardinal;
var CacheLen: Cardinal;
begin
  if BlockSize <> 0 then
  begin
    CacheLen := Length(FCache);
    SetLength(FCache, CacheLen + BlockSize);
    Move(Block^, FCache[CacheLen+1], BlockSize);
    Result := Length(FCache);
  end else
    Result := 0;
end;

procedure TDnTcpChannel.InsertToCache(Block: PChar; BlockSize: Cardinal);
var CacheLen : Cardinal;
begin
  if (Block<>Nil) and (BlockSize <> 0) then
  begin
    CacheLen := Length(FCache);
    SetLength(FCache, CacheLen + BlockSize);
    if CacheLen <> 0 then
      Move(FCache[1], FCache[1+BlockSize], CacheLen);
    Move(Block^, FCache[1], BlockSize);
  end;
end;


function TDnTcpChannel.ExtractFromCache(Block: PChar; BlockSize: Cardinal): Cardinal;
var CacheLen: Cardinal;
begin
  CacheLen := Length(FCache);
  if CacheLen < BlockSize then
  begin
    Move(FCache[1], Block^, CacheLen);
    SetLength(FCache, 0);
    Result := CacheLen;
  end else
  begin
    Move(FCache[1], Block^, BlockSize);
    Delete(FCache, 1, BlockSize);
    Result := BlockSize;
  end;
end;

function TDnTcpChannel.CacheHasData: Boolean;
begin
  Result := Length(FCache) <> 0;
end;

//Stop/cancel the timeout tracking activity. It should be called only 1 time.
//There isn't a back way - you can't start timeout tracking again. 
procedure TDnTcpChannel.StopTimeOutTracking;
begin
  if FTimeOutObserver <> Nil then
  begin
    FTimeOutAbort := True;
    FTimeOutObserver.Cancel;
    FTimeOutObserver := Nil;
  end;
end;

function TDnTcpChannel.IsClosing: Boolean;
begin
  Result := FClosingRequest <> Nil;
end;

procedure TDnTcpChannel.RunRequest(Request: IDnIORequest);
begin
  if not FClosing then
  begin
    FRunGuard.Acquire;
    try
      case Request.RequestType of
        rtClose:    begin
                      FClosingRequest := Request;
                      if (FReadQueue.Count = 0) and (FWriteQueue.Count = 0) and
                          (FConnectingRequest = Nil) then
                        Request.Execute;
                    end;
        rtBrutalClose:
                    begin
                      if FClosingRequest = Nil then
                      begin
                        FClosingRequest := Request;
                        Request.Execute;
                      end;
                    end;

        rtConnect:  begin
                      FConnectingRequest := Request;
                      Request.Execute;
                    end;

        rtRead:     if (FReadQueue.Add(Request) = 0) and (FConnectingRequest = Nil) then
                      Request.Execute;

        rtWrite:    if (FWriteQueue.Add(Request) = 0) and (FConnectingRequest = Nil) then
                      Request.Execute;
      end;

    finally
      FRunGuard.Release;
    end;
  end
  else
    raise EDnException.Create(ErrChannelClosing, 0);
end;

procedure TDnTcpChannel.Bind(Tracker: Pointer);
begin
  FTracker := Tracker;
end;

procedure TDnTcpChannel.Unbind;
begin
  FTracker := Nil;
end;

function TDnTcpChannel.Tracker: Pointer;
begin
  Result := FTracker;
end;

function  TDnTcpChannel.IsBound: Boolean;
begin
  Result := FTracker <> Nil;
end;

function TDnTcpChannel.RemotePort: Word;
begin
  Result := ntohs(FRemoteAddr.sin_port);
end;

function TDnTcpChannel.RemoteAddr: String;
begin
  Result := StrPas(inet_ntoa(FRemoteAddr.sin_addr));
end;

function TDnTcpChannel.RemoteHost: String;
begin
  Result := StrPas(gethostbyaddr(@FRemoteAddr.sin_addr, SizeOf(FRemoteAddr),AF_INET)^.h_name);
end;

procedure TDnTcpChannel.CloseSocketHandle;
var TempSocket: TSocket;
begin
  //stop timeouts
  StopTimeOutTracking;
  TempSocket := FSocket;
  FSocket := INVALID_SOCKET;
  Winsock2.closesocket(TempSocket);
end;

class function TDnTcpChannel.MatchingRequest(Context: Pointer): TDnTcpRequest;
var PContext: PDnReqContext;
begin
  PContext := PDnReqContext(Context);
  Result := TDnTcpRequest(PContext^.FRequest);
end;

function TDnTcpChannel.IsClosed: Boolean;
begin
  Result := FSocket = INVALID_SOCKET;
end;

procedure TDnTcpChannel.SetNagle(Value: Boolean);
var Temp: LongBool;
begin
  Temp := Value;
  Winsock2.setsockopt(FSocket, IPPROTO_TCP, TCP_NODELAY, PChar(@Temp), SizeOf(Temp));
end;

procedure TDnTcpChannel.ExecuteNext(Request: IDnIORequest);
begin
  try
    FRunGuard.Acquire;

    if Request.IsCPUNeeded then
      Exit;
    case Request.RequestType of
      rtBrutalClose,
      rtClose:    begin
                    FClosingRequest := Nil;
                  end;
      rtRead:     begin
                    if FReadQueue[0] = IUnknown(Request) then
                      FReadQueue.Delete(0);
                    if FReadQueue.Count <> 0 then
                      (FReadQueue[0] as IDnIORequest).Execute;
                  end;
      rtWrite:    begin
                    if FWriteQueue[0] = IUnknown(Request) then
                      FWriteQueue.Delete(0);
                    if FWriteQueue.Count <> 0 then
                      (FWriteQueue[0] as IDnIORequest).Execute;
                  end;
      rtConnect:  begin
                    FConnectingRequest := Nil;
                    if FReadQueue.Count <> 0 then
                      (FReadQueue[0] as IDnIORequest).Execute;
                    if FWriteQueue.Count <> 0 then
                      (FWriteQueue[0] as IDnIORequest).Execute;
                    if (FReadQueue.Count = 0) and (FWriteQueue.Count = 0) and
                        (FClosingRequest <> Nil) then
                      FClosingRequest.Execute;
                  end;
    end;
  finally
    FRunGuard.Release;
  end;
end;

destructor TDnTcpChannel.Destroy;
begin
  FreeAndNil(FRunGuard);
  FreeAndNil(FTimeOutGuard);
  //FreeAndNil(FRequest);
  FreeAndNil(FReadQueue);
  FreeAndNil(FWriteQueue);
  if FClosingRequest <> Nil then
    FClosingRequest := Nil;
//  if FConnectRequest <> Nil then
//    FConnectRequest := Nil;
  inherited Destroy;
end;
//-----------------------------------------------------------------------------

constructor TDnTcpReactor.Create{$IFDEF ROOTISCOMPONENT}(AOwner: TComponent){$ENDIF};
begin
  inherited Create{$IFDEF ROOTISCOMPONENT}(AOwner){$ENDIF};
  FPort := 0;
  FActive := False;
  FGuard := TDnMutex.Create;
  FChannels := TInterfaceList.Create;
  FExecutor := Nil;
  FTimer := Nil;
  FTimerExecutor := Nil;
  FLogger := Nil;
  FLogLevel := llMandatory;
  FTimeOutExpiredEvent := Nil;
end;

destructor TDnTcpReactor.Destroy;
begin
  if FActive then
    SetActive(False);
  FreeAndNil(FGuard);
  inherited Destroy;
end;

function  TDnTcpReactor.GetChannelCount: Integer;
begin
  FGuard.Acquire;
  if Assigned(FChannels) then
    Result := FChannels.Count
  else
    Result := 0;
  FGuard.Release;
end;

{$IFDEF ROOTISCOMPONENT}
procedure TDnTcpReactor.Notification(AComponent: TComponent; Operation: TOperation);
begin
  if Operation = opRemove then
  begin
    if AComponent = FLogger then
      FLogger := Nil
    else
    if AComponent = FExecutor then
      FExecutor := Nil;
  end;
end;
{$ENDIF}


procedure TDnTcpReactor.SetActive(Value: Boolean);
begin
  if not FActive and Value then
    FActive := TurnOn
  else if FActive and not Value then
    FActive := TurnOff;
end;

function TDnTcpReactor.MakeChannel(const IPAddress: String; Port: Word): IDnChannel;
begin
  Result := TDnTcpChannel.CreateEmpty(Self, IPAddress, Port);
{
  FGuard.Acquire;
  try
    FChannels.Add(Result);
  finally
    FGuard.Release;
  end; }
end;

procedure TDnTcpReactor.SetTimeout(Channel: IDnChannel; Value: Cardinal);
var ChannelImpl: TDnTcpChannel;
begin
  if @FTimeOutExpiredEvent = Nil then
    raise EDnException.Create(ErrInvalidConfig, 0);
  ChannelImpl := TDnTcpChannel.CheckImpl(Channel);
  ChannelImpl.SetTimeOut(Value);
  Channel := Nil;
end;


procedure TDnTcpReactor.RemoveChannel(Channel: IDnChannel);
begin
  if Channel.IsClosed then
  begin
    FGuard.Acquire;
    FChannels.Remove(Channel);
    FGuard.Release;
    Channel := Nil;
  end
  else
    raise EDnException.Create(ErrCannotRemoveOpenedChannel, 0);
end;

procedure TDnTcpReactor.TimeOutFired(Context: TDnThreadContext; Channel: IDnChannel;
                              ExpiredTacts: Cardinal; Key: Pointer);
var ChannelImpl: TDnTcpChannel;
begin
  ChannelImpl := TDnTcpChannel.CheckImpl(Channel);
  if ChannelImpl.FFinishedIOCount = Cardinal(Key) then
    PostQueuedCompletionStatus(FPort, $FFFFFFFF, 0, Pointer(ChannelImpl))
  else
  if ChannelImpl.FTimeOut <> 0 then
    ChannelImpl.IssueTimeOutRequest(ChannelImpl.TimeOut, Nil)
  else
  begin
    ChannelImpl.FFirstTimeOutRequest := False;
    ChannelImpl.FFinishedIOCount := 0;
  end;
end;


function TDnTcpReactor.TurnOn: Boolean;
var TempSocket: TSocket;
begin
  TempSocket := Winsock2.socket(AF_INET, SOCK_STREAM, IPPROTO_IP);
  if TempSocket = Winsock2.INVALID_SOCKET then
    raise EDnException.Create(ErrWin32Error, WSAGetLastError(), 'Socket');

  FPort := CreateIOCompletionPort(TempSocket, 0, 0, 1);
  if FPort = 0 then
  begin
    Winsock2.closesocket(TempSocket);
    raise EDnException.Create(ErrWin32Error, GetLastError(), 'CreateIOCompletionPort');
  end;
  Winsock2.closesocket(TempSocket);


  FTimer := TDnTimer.Create({$IFDEF ROOTISCOMPONENT}Nil{$ENDIF});
  FTimerExecutor := TDnSimpleExecutor.Create({$IFDEF ROOTISCOMPONENT}Nil{$ENDIF});
  FTimer.Executor := FTimerExecutor;
  FTimer.OnTimerNotify := TimeOutFired;
  FTimer.Active := True;

  FThread := TDnTcpReactorThread.Create(Self);

  Result := True;
end;

procedure TDnTcpReactor.CloseChannels;
var i: Integer;
    ChannelImpl: TDnTcpChannel;
begin
  FGuard.Acquire;
  for i:=0 to FChannels.Count-1 do
  begin
    ChannelImpl := TDnTcpChannel.CheckImpl(FChannels[i]);
    ChannelImpl.CloseSocketHandle;
  end;
  FGuard.Release;
end;

function TDnTcpReactor.TurnOff: Boolean;
var ChannelImpl: TDnTcpChannel;
    i: Integer;
begin
  FGuard.Acquire;
  for i:=0 to FChannels.Count-1 do
  begin
    ChannelImpl := TDnTcpChannel.CheckImpl(FChannels[i]);
    ChannelImpl.CloseSocketHandle;
  end;
  PostQueuedCompletionStatus(FPort, 0, 0, Nil);
  FreeAndNil(FThread);
  FreeAndNil(FTimer);
  FreeAndNil(FTimerExecutor);
  //FreeAndNil(FTimerEngine);
  FExecutor.Active := False;
  //for i:=0 to FChannels.Count-1 do
  //  TDnTcpChannel.(FChannels[i])._Release;
  FChannels.Clear;
  FGuard.Release;
  Result := False;
end;

procedure TDnTcpReactor.PostChannel(Channel: IDnChannel);
var ChannelImpl: TDnTcpChannel;
begin
  ChannelImpl := TDnTcpChannel.CheckImpl(Channel);
  if CreateIOCompletionPort(ChannelImpl.FSocket, FPort, Cardinal(Pointer(ChannelImpl)), 1) = 0 then
    raise EDnException.Create(ErrWin32Error, GetLastError(), 'CreateIOCompletionPort');
  FChannels.Add(Channel);
end;

procedure TDnTcpReactor.PostChannelError(Channel: TDnTcpChannel; Request: TDnTcpRequest);
begin
  Request.FErrorCode := WSAGetLastError;
  //PostQueuedCompletionStatus(FPort,
  FExecutor.PostEvent(Request);//, Channel);
end;

procedure TDnTcpReactor.OnTimeOutExpired(Context: TDnThreadContext; Channel: IDnChannel);
var ChannelImpl: TDnTcpChannel;
begin
  ChannelImpl := TDnTcpChannel.CheckImpl(Channel);
  if not ChannelImpl.FTimeOutAbort then
    FTimeOutExpiredEvent(Context, Channel);
end;


//-------------------------------------------------------------------------
constructor TDnTcpReactorThread.Create(Reactor: TDnTcpReactor);
begin
  inherited Create;
  FReactor := Reactor;
  FreeOnTerminate := False;
  Self.Run;
end;

destructor TDnTcpReactorThread.Destroy;
begin
  inherited Destroy;
end;

procedure TDnTcpReactorThread.CreateContext;
begin
  FContext := Nil;
  SetCurrentContext(Nil);
end;

procedure TDnTcpReactorThread.DestroyContext;
begin
  ;
end;

procedure TDnTcpReactorThread.LogMessage(S: String);
begin
  if FReactor.FLogger <> Nil then
  try
    FReactor.FLogger.LogMsg(FReactor.FLogLevel, S);
  except
    ; //if logger failed - just ignore
  end;
end;

procedure TDnTcpReactorThread.DoRequest(Request: TDnTcpRequest; Channel: TDnTcpChannel);
var Response: IDnIOResponse;
begin
  Response := Request;
  Channel.ExecuteNext(Request);
  if Assigned(FReactor.FExecutor) then
    FReactor.FExecutor.PostEvent(Response);
  Response := Nil;
  Request := Nil;
end;

procedure TDnTcpReactorThread.DoClose(Channel: TDnTcpChannel; Request: TDnTcpRequest);
begin
end;

procedure TDnTcpReactorThread.ParseIONotification(Transferred: Cardinal; Key: Cardinal; Overlapped: POverlapped);
var Channel: TDnTcpChannel;
    Request: TDnTcpRequest;
    ReqContext: PDnReqContext;
begin
  InterlockedIncrement(ParseIONotificationCount);
  if Transferred = $FFFFFFFF then
  begin
    Channel := TDnTcpChannel(Overlapped);
    Channel.FTimeOutExpired := True;
    if not Channel.FTimeOutAbort then
      FReactor.FExecutor.PostEvent(TDnTimeOutWrapper.Create(Channel, FReactor));
  end else
  begin
    ReqContext := PDnReqContext(Overlapped);
    if not ReqContext^.FReqRouting then
    begin
      FReactor.FExecutor.PostEvent(IUnknown(ReqContext^.FRequest) as IDnIOResponse);
    end else
    begin
      Request := TDnTcpChannel.MatchingRequest(Overlapped);
      Channel := TDnTcpChannel.CheckImpl(Request.FChannel);
      UpdateCounters(Channel, Request);
      Request.SetTransferred(Transferred);
      if not Channel.TimeOutExpired or (Request.RequestType = rtBrutalClose) then
      begin
        if not Request.IsComplete and not Request.IsCPUNeeded then
        begin
          if Transferred <> 0 then
            Request.ReExecute
          else
            raise EDnException.Create(ErrZeroTransferDetected, 0);
        end else
          DoRequest(Request, Channel);
      end;
    end; //if not ReqContext.FReqRouting
  end;
end;

procedure TDnTcpReactorThread.ParseIOError(Transferred: Cardinal; Key: Cardinal; Overlapped: POverlapped);
var Channel: TDnTcpChannel;
    Request: TDnTcpRequest;
    ReqContext: PDnReqContext;
    Source: IUnknown;
begin
  ReqContext := PDnReqContext(Overlapped);
  if ReqContext^.FReqRouting then
  begin
    Request := TDnTcpChannel.MatchingRequest(Overlapped);
    Channel := TDnTcpChannel.CheckImpl(Request.FChannel);
    Channel.StopTimeOutTracking; //it is really fast operation
    UpdateCounters(Channel, Request);
    Request.CatchError;
    DoRequest(Request, Channel);
  end else
  begin
    Source := IUnknown(ReqContext^.FRequest);
    (Source as IDnIOErrorValue).StoreError(WSAGetLastError());
    FReactor.FExecutor.PostEvent(Source as IDnIOResponse);
  end;
end;

procedure TDnTcpReactorThread.UpdateCounters(Channel: TDnTcpChannel; Request: TDnTcpRequest);
begin
  Channel.IncrementIOCounter;
{  case Request.RequestType of
    rtRead:       Channel.IncrementReadCounter;
    rtWrite:      Channel.IncrementWriteCounter;
    rtConnect:    Channel.IncrementReadCounter;
    rtClose:      Channel.IncrementReadCounter;
  end; }
end;

procedure TDnTcpReactorThread.ThreadRoutine;
var Transferred: Cardinal;
    Key: Cardinal;
    Overlapped: POverlapped;
    ResCode: LongBool;
begin
  SetCurrentContext(Nil);
  while True do
  begin
    ResCode := GetQueuedCompletionStatus(FReactor.FPort, Transferred, Key, Overlapped, INFINITE);
    if ResCode then
    begin
      if (Transferred = 0) and (Key = 0) and (Overlapped = Nil) then
        Exit //signal to terminate thread
      else
      begin
        ; //notify channel
        ParseIONotification(Transferred, Key, Overlapped);
      end;
    end
    else //error trapped
    begin
      if Overlapped = Nil then
         //win32 kernel error? just ignore it 8)
      else
      begin
        ; //IO error
        ParseIOError(Transferred, Key, Overlapped);
      end;
    end;
  end;
end;
//-------------------------------------------------------------------------


procedure Register;
begin
  {$IFDEF ROOTISCOMPONENT}
  RegisterComponents('DNet', [TDnTcpReactor]);
  {$ENDIF}
end;

initialization
  ParseIONotificationCount := 0;
end.
