// The contents of this file are used with permission, subject to
// the Mozilla Public License Version 1.1 (the "License"); you may
// not use this file except in compliance with the License. You may
// obtain a copy of the License at
// http://www.mozilla.org/MPL/MPL-1.1.html
//
// Software distributed under the License is distributed on an
// "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
// implied. See the License for the specific language governing
// rights and limitations under the License.
unit DnTcpRequests;
interface
uses
  Classes, SysUtils, Windows, Winsock, Winsock2,
  DnTcpReactor, DnConst, DnInterfaces, DnRtl;

type

  IDnTcpCloseHandler = interface
  ['{AB1279A1-BBC9-11d5-BDB9-0000212296FE}']
    procedure DoClose(Context: TDnThreadContext; Channel: IDnChannel; Key: Pointer);
    procedure DoCloseError(Context: TDnThreadContext; Channel: IDnChannel; Key: Pointer;
                                  ErrorCode: Cardinal);
  end;

  TDnTcpCloseRequest = class (TDnTcpRequest)
  protected
    FWSABuf:      WSABUF;
    FTempBuffer:  String;
    FRead:        Cardinal;
    FFlags:       Cardinal;
    FHandler:     IDnTcpCloseHandler;
    FBrutal:      Boolean;
    procedure     SetTransferred(Transferred: Cardinal); override;
    
    //IDnIORequest
    procedure Execute; override;
    function  IsComplete: Boolean; override;
    procedure ReExecute; override;
    function  RequestType: TDnIORequestType; override;
    function  IsCPUNeeded: Boolean; override;

    //IDnIOResponse
    procedure CallHandler(Context: TDnThreadContext); override;

  public
    constructor Create(Channel: IDnChannel; Key: Pointer; Handler: IDnTcpCloseHandler;
                        Brutal: Boolean = False);
    destructor  Destroy; override;
  end;

  IDnTcpReadHandler = interface
  ['{AB1279A2-BBC9-11d5-BDB9-0000212296FE}']
    procedure DoRead(Context: TDnThreadContext; Channel: IDnChannel; Key: Pointer;
                                  Buf: PChar; BufSize: Cardinal);
    procedure DoReadError(Context: TDnThreadContext; Channel: IDnChannel; Key: Pointer;
                                  ErrorCode: Cardinal);
    procedure DoReadClose(Context: TDnThreadContext; Channel: IDnChannel; Key: Pointer);
  end;

  TDnTcpReadRequest = class (TDnTcpRequest)
  protected
    FWSABuf:  WSABUF;
    FRead:    Cardinal;
    FToRead:  Cardinal;
    FFlags:   Cardinal;
    FHandler: IDnTcpReadHandler;
    FMustAll: Boolean;
    FStrBuf:  String;
    
    procedure SetTransferred(Transferred: Cardinal); override;

    //IDnIORequest
    procedure Execute; override;
    function IsComplete: Boolean; override;
    procedure ReExecute; override;
    function  RequestType: TDnIORequestType; override;
    function  IsCPUNeeded: Boolean; override;
    //IDnIOResponse
    procedure CallHandler(Context: TDnThreadContext); override;
  public
    constructor Create( Channel: IDnChannel; Key: Pointer;
                        Handler: IDnTcpReadHandler; Buf: PChar;
                        BufSize: Cardinal; MustAll:  Boolean = True);// overload;
    constructor CreateString( Channel:  IDnChannel; Key: Pointer;
                        Handler: IDnTcpReadHandler; BufSize: Integer;
                        MustAll:  Boolean = True); 
    destructor Destroy; override;
    procedure Init( Channel: IDnChannel; Key: Pointer;
                    Handler: IDnTcpReadHandler; Buf: PChar;
                    BufSize: Cardinal; MustAll: Boolean = True);

  end;

  IDnTcpLineHandler = interface
  ['{AB1279A4-BBC9-11d5-BDB9-0000212296FE}']
    procedure DoLine( Context: TDnThreadContext; Channel: IDnChannel; Key: Pointer;
                      ReceivedLine: String; EolFound: Boolean );
    procedure DoLineError(Context: TDnThreadContext; Channel: IDnChannel; Key: Pointer;
                          ErrorCode: Cardinal);
    procedure DoLineClose(Context: TDnThreadContext; Channel: IDnChannel; Key: Pointer);
  end;

  TDnTcpLineRequest = class (TDnTcpRequest)
  protected
    FWSABuf:    WSABUF;
    FRead:      Cardinal;
    FToRead:    Integer;
    FWasRead:   Integer;
    FTotalWasRead:
                Integer;
    FFlags:     Cardinal;
    FHandler:   IDnTcpLineHandler;
    FMaxSize:   Integer;
    FRecv:      String;
    FEolFound:  Boolean;
    FEolSign:   PChar;

    FBufInitialSize: Integer;
    FBufGranularity: Integer;
    
    function  CheckForEol(Line: PChar; Len: Integer): Integer;
    procedure SetTransferred(Transferred: Cardinal); override;
    function  IssueWSARecv( s : TSocket; lpBuffers : LPWSABUF; dwBufferCount : DWORD; var lpNumberOfBytesRecvd : DWORD; var lpFlags : DWORD;
              lpOverlapped : LPWSAOVERLAPPED; lpCompletionRoutine : LPWSAOVERLAPPED_COMPLETION_ROUTINE ): Integer; stdcall;
    procedure Reset(Channel: IDnChannel; Key: Pointer;
                        Handler: IDnTcpLineHandler; MaxSize: Cardinal);
    //IDnIORequest
    procedure Execute; override;
    function  IsComplete: Boolean; override;
    procedure ReExecute; override;
    function  RequestType: TDnIORequestType; override;
    function  IsCPUNeeded: Boolean; override;

    //IDnIOResponse
    procedure CallHandler(Context: TDnThreadContext); override;
  public
    constructor Create( Channel: IDnChannel; Key: Pointer;
                        Handler: IDnTcpLineHandler; MaxSize: Cardinal );
    destructor Destroy; override;
  end;

  IDnTcpWriteHandler = interface
  ['{AB1279A5-BBC9-11d5-BDB9-0000212296FE}']
    procedure DoWrite(Context: TDnThreadContext; Channel: IDnChannel; Key: Pointer;
                      Buf: PChar; BufSize: Cardinal);
    procedure DoWriteError( Context: TDnThreadContext; Channel: IDnChannel; Key: Pointer;
                            ErrorCode: Cardinal );
  end;


  TDnTcpWriteRequest = class (TDnTcpRequest)
  protected
    FWSABuf:        WSABUF;
    FWritten:       Cardinal;
    FToWrite:       Cardinal;
    FFlags:         Cardinal;
    FHandler:       IDnTcpWriteHandler;
    FTempStorage:   String;

    procedure SetTransferred(Transferred: Cardinal); override;
    //IDnIORequest
    procedure Execute; override;
    function  IsComplete: Boolean; override;
    procedure ReExecute; override;
    function  RequestType: TDnIORequestType; override;
    function  IsCPUNeeded: Boolean; override;

    //IDnIOResponse
    procedure CallHandler(Context: TDnThreadContext); override;
  public
    constructor Create( Channel: IDnChannel; Key: Pointer; Handler: IDnTcpWriteHandler;
                        Buf: PChar; BufSize: Cardinal); overload;
    constructor CreateString( Channel: IDnChannel; Key: Pointer; Handler: IDnTcpWriteHandler;
                        Buf: String); overload;
    destructor Destroy; override;
  end;



implementation
var
  CRLFZero:PChar = #13#10#0;


//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

constructor TDnTcpCloseRequest.Create(Channel: IDnChannel; Key: Pointer;
                                      Handler: IDnTcpCloseHandler; Brutal: Boolean);
begin
  inherited Create(Channel, Key);
  FHandler := Handler;
  SetLength(FTempBuffer, 1024);
  FWSABuf.len := 1024;
  FWSABuf.buf := @FTempBuffer[1];
  FBrutal := Brutal;
end;

function TDnTcpCloseRequest.RequestType: TDnIORequestType;
begin
  if FBrutal then
    Result := rtBrutalClose
  else
    Result := rtClose;
end;

procedure TDnTcpCloseRequest.SetTransferred(Transferred: Cardinal);
begin
  FRead := Transferred;
  FWSABuf.buf := PChar(FTempBuffer);
  FWSABuf.len := 1024;
end;

procedure TDnTcpCloseRequest.Execute;
var ResCode: Integer;
    ChannelImpl: TDnTcpChannel;
begin
  inherited Execute;
  ChannelImpl := TDnTcpChannel.CheckImpl(FChannel);
  Winsock2.shutdown(ChannelImpl.SocketHandle, SD_SEND); //disable sending
  InterlockedIncrement(PendingRequests);
  if not FBrutal then
  begin
    FRead := 0;
    ResCode := Winsock2.WSARecv(ChannelImpl.SocketHandle, @FWSABuf, 1,  FRead, FFlags, @FContext, Nil);
    if ResCode <> 0 then
    begin
      ResCode := WSAGetLastError;
      if ResCode <> WSA_IO_PENDING then
        Self.PostError(ResCode, 0);
    end;
  end else
    PostQueuedCompletionStatus(ChannelImpl.Reactor.PortHandle, 0,
                                Cardinal(Pointer(ChannelImpl)), @FContext);
end;

procedure TDnTcpCloseRequest.ReExecute;
begin
  Execute;
end;

function TDnTcpCloseRequest.IsComplete: Boolean;
var ChannelImpl: TDnTcpChannel;
begin
  inherited IsComplete;
  ChannelImpl := TDnTcpChannel.CheckImpl(FChannel);
  Result := (FRead = 0) or (FErrorCode <> 0);
  if Result then
  begin
    ChannelImpl.CloseSocketHandle;
  end;
end;

function TDnTcpCloseRequest.IsCPUNeeded: Boolean;
begin
  Result := False;
end;

procedure TDnTcpCloseRequest.CallHandler(Context: TDnThreadContext);
var ChannelImpl: TDnTcpChannel;
begin
  try
    ChannelImpl := TDnTcpChannel.CheckImpl(FChannel);
    if FErrorCode <> 0 then
      FHandler.DoCloseError(Context, FChannel, FKey, FErrorCode)
    else
      FHandler.DoClose(Context, FChannel, FKey);
    //ChannelImpl.CloseSocketHandle;
    //drop channel
    ChannelImpl.Reactor.RemoveChannel(FChannel);
  finally
    //InterlockedDecrement(PendingRequests);
  end;
end;


destructor  TDnTcpCloseRequest.Destroy;
begin
  FHandler := Nil;
  inherited Destroy;
end;
//-------------------------------------------------------------------------------------------------

constructor TDnTcpReadRequest.Create( Channel: IDnChannel; Key: Pointer;
                                      Handler: IDnTcpReadHandler; Buf: PChar;
                                      BufSize: Cardinal; MustAll: Boolean = True );
begin
  inherited Create(Channel, Key);
  SetLength(FStrBuf, 0);
  Init(Channel, Key, Handler, Buf, BufSize, MustAll);
end;

constructor TDnTcpReadRequest.CreateString( Channel: IDnChannel; Key: Pointer;
                                      Handler: IDnTcpReadHandler; BufSize: Integer;
                                      MustAll: Boolean = True );
begin
  inherited Create(Channel, Key);
  SetLength(FStrBuf, BufSize);
  Init(Channel, Key, Handler, @FStrBuf[1], Length(FStrBuf), MustAll);
end;

procedure TDnTcpReadRequest.Init(Channel: IDnChannel; Key: Pointer;
                                  Handler: IDnTcpReadHandler; Buf: PChar;
                                  BufSize: Cardinal; MustAll: Boolean = True);
begin
  FWSABuf.Len := BufSize;
  FWSABuf.Buf := Buf;
  FRead := 0;
  FToRead := BufSize;
  FFlags := 0;
  //FTotalSize := BufSize;
  FStartBuffer := @Buf;
  FHandler := Handler;
  FMustAll := MustAll;
end;

destructor TDnTcpReadRequest.Destroy;
begin
  FHandler := Nil;
  inherited Destroy;
end;

function  TDnTcpReadRequest.RequestType: TDnIORequestType;
begin
  Result := rtRead;
end;

procedure TDnTcpReadRequest.SetTransferred(Transferred: Cardinal);
begin
  FRead := Transferred;
  Inc(FWSABuf.buf, FRead);
  Dec(FWSABuf.len, FRead);
end;

procedure TDnTcpReadRequest.Execute;
var ChannelImpl: TDnTcpChannel;
    ResCode: Integer;
begin
  inherited Execute;
  ChannelImpl := TDnTcpChannel.CheckImpl(FChannel);
  //check the channel read cache
  FRead := ChannelImpl.ExtractFromCache(FWSABuf.buf, FToRead);
  //inc pending requests count
  InterlockedIncrement(PendingRequests);
  if FRead = FToRead then
    PostQueuedCompletionStatus(ChannelImpl.Reactor.PortHandle, FRead,
                                Cardinal(Pointer(ChannelImpl)), @FContext)
  else
  begin //not read yet...
    Inc(FWSABuf.buf, FRead);
    Dec(FWSABuf.len, FRead);
    ResCode := Winsock2.WSARecv(ChannelImpl.SocketHandle, @FWSABuf, 1,  FRead, FFlags, @FContext, Nil);
    //ResCode := Integer(ReadFileEx(ChannelImpl.SocketHandle, @FWSABuf, FRead, @FContext, Nil));
    if ResCode <> 0 then
    begin
      ResCode := WSAGetLastError;
      if ResCode <> WSA_IO_PENDING then
        Self.PostError(ResCode, FRead)
      else
        ;
    end;
  end;
end;

function TDnTcpReadRequest.IsComplete: Boolean;
begin
  inherited IsComplete;

  Result := (FWSABuf.len = 0) or    //everything is read
            (FRead = 0) or          //client close
            (FErrorCode <> 0) or    //network error
            not FMustAll;           //raw read
  if (FRead = 0) or (FErrorCode <> 0) then
    TDnTcpChannel.CheckImpl(FChannel).StopTimeOutTracking;
end;

function TDnTcpReadRequest.IsCPUNeeded: Boolean;
begin
  Result := False;
end;

procedure TDnTcpReadRequest.ReExecute;
begin
//  Dec(FWSABuf.len, FRead);
//  Inc(FWSABuf.buf, FRead);
  FRead := 0;
  Execute;
end;

procedure TDnTcpReadRequest.CallHandler(Context: TDnThreadContext);
begin
  try
    if (FErrorCode = 0) and (FRead <> 0) then
      FHandler.DoRead(Context, FChannel, FKey, FStartBuffer, FToRead - FWSABuf.len)
    else
    if FRead = 0 then
      FHandler.DoReadClose(Context, FChannel, FKey)
    else
      FHandler.DoReadError(Context, FChannel, FKey, FErrorCode);
  finally
    //InterlockedDecrement(RequestsPending);
  end;
end;
//-----------------------------------------------------------------------------

constructor TDnTcpLineRequest.Create( Channel: IDnChannel; Key: Pointer;
                                      Handler: IDnTcpLineHandler; MaxSize: Cardinal );
begin
  inherited Create(Channel, Key);
  if MaxSize > 1024 then
    FBufInitialSize := 1024
  else
    FBufInitialSize := MaxSize;
  
  Reset(Channel, Key, Handler, MaxSize);
  FContext.FRequest := Pointer(Self);
  SetLength(FRecv, MaxSize);
  FWSABuf.Len := MaxSize;
  FWSABuf.Buf := PChar(FRecv);
  FRead := 0;
  FToRead := MaxSize;
  FFlags := 0;
  //FTotalSize := MaxSize;
  FStartBuffer := PChar(FRecv);
  
  FHandler := Handler;
  FMaxSize := MaxSize;
  FEolFound := False;
  FWasRead := 0;
  FEolSign := CRLFZero; //CRLF - zero terminated string
end;

const MaxSizePerRecv: Cardinal = 65536;

procedure TDnTcpLineRequest.Reset(Channel: IDnChannel; Key: Pointer;
                                  Handler: IDnTcpLineHandler; MaxSize: Cardinal);
begin
  //bind POverlapped to this object
  FContext.FRequest := Pointer(Self);
  //allocate memory for recv'ed data
  SetLength(FRecv, FBufInitialSize);//SetLength(FRecv, MaxSize);
  FWSABuf.Len := FBufInitialSize; //FWSABuf.Len := MaxSize;
  FToRead := FBufInitialSize; //FToRead := MaxSize;
  FWSABuf.Buf := PChar(FRecv);
  FRead := 0;
  FFlags := 0;
  FStartBuffer := PChar(FRecv);
  FHandler := Handler;
  FMaxSize := MaxSize;
  FEolFound := False;
  FWasRead := 0;
  FEolSign := CRLFZero; //CRLF - zero terminated string
  FTotalWasRead := 0;
end;

destructor TDnTcpLineRequest.Destroy;
begin
  FHandler := Nil;
  inherited Destroy;
end;

function  TDnTcpLineRequest.RequestType: TDnIORequestType;
begin
  Result := rtRead;
end;

//scans Line for CRLF sequence
function TDnTcpLineRequest.CheckForEol(Line: PChar; Len: Integer): Integer;
var Ptr: PChar;
begin
  Ptr := StrPos(Line, FEolSign);
  if Ptr <> Nil then
    Result := Ptr - Line + 2
  else
    Result := -1;
end;

procedure TDnTcpLineRequest.SetTransferred(Transferred: Cardinal);
begin
  FRead := Transferred;
  Dec(FWSABuf.len, FRead);
  Inc(FWSABuf.buf, FRead);
  Inc(FTotalWasRead, FRead); 
end;

function  TDnTcpLineRequest.IssueWSARecv( s : TSocket; lpBuffers : LPWSABUF; dwBufferCount : DWORD; var lpNumberOfBytesRecvd : DWORD; var lpFlags : DWORD;
              lpOverlapped : LPWSAOVERLAPPED; lpCompletionRoutine : LPWSAOVERLAPPED_COMPLETION_ROUTINE ): Integer;
begin
  Result := Winsock2.WSARecv(s, lpBuffers, dwBufferCount, lpNumberOfBytesRecvd, lpFlags, lpOverlapped, lpCompletionRoutine);
end;


procedure TDnTcpLineRequest.Execute;
var ResCode: Integer;
    Len: Cardinal;
    ChannelImpl: TDnTcpChannel;
begin
  inherited Execute;
  FContext.FRequest := Self;
  //grab the channel read cache
  InterlockedIncrement(PendingRequests);
  ChannelImpl := TDnTcpChannel.CheckImpl(FChannel);
  if ChannelImpl.CacheHasData then
  begin
    FRead := ChannelImpl.ExtractFromCache(FWSABuf.buf, FWSABuf.len);
    Len := Self.CheckForEol(FWSABuf.buf, FRead);
    if Len <> $FFFFFFFF then
    begin
      ChannelImpl.InsertToCache(FWSABuf.buf + Len, FRead - Len);
      FRead := Len;
      PostQueuedCompletionStatus(ChannelImpl.Reactor.PortHandle, FRead, Cardinal(Pointer(ChannelImpl)), @FContext);
    end else
    if FRead <> 0 then
      PostQueuedCompletionStatus(ChannelImpl.Reactor.PortHandle, FRead, Cardinal(Pointer(ChannelImpl)), @FContext)
  end
  else
  begin //start reading from socket
    ResCode := IssueWSARecv(ChannelImpl.SocketHandle, @FWSABuf, 1,  FRead, FFlags, @FContext, Nil);

    if ResCode <> 0 then
    begin
      ResCode := WSAGetLastError;
      if ResCode <> WSA_IO_PENDING then
        Self.PostError(ResCode, FRead);
    end;
  end;
end;

function TDnTcpLineRequest.IsComplete: Boolean;
var ChannelImpl: TDnTcpChannel;
    Found: Integer;
    Tail: Integer;
    NeedToRead: Cardinal;
    OldSize, NewSize: Integer;
begin
  inherited IsComplete;
  ChannelImpl := TDnTcpChannel.CheckImpl(FChannel);
  if (FErrorCode <> 0) or (FRead = 0)then
  begin
    ChannelImpl.StopTimeOutTracking;
    Result := True;
    Exit;
  end;
  FEolFound := False;
  Dec(FToRead, FRead);

  Inc(FWasRead, FRead);
  //Inc(FTotalWasRead, FRead);
  if FWasRead <> 0 then
    Found := Self.CheckForEol(PChar(FRecv), FWasRead)
  else
    Found := -1;

  if (Found = -1) and (FToRead <> 0) then
    Result := False
  else
  if (Found = -1) and (FToRead = 0) then
  begin //ok, here we read all FToRead's bytes but EOL is not found
    //ok, do we need to read smth else?
    if Length(FRecv) < FMaxSize then
    begin
      OldSize := Length(FRecv);
      NewSize := Trunc(Length(FRecv) * 1.25 + 0.5);
      if NewSize > FMaxSize then
        NewSize := FMaxSize;
      SetLength(FRecv, NewSize);
      NeedToRead := NewSize - OldSize;
      FWSABuf.buf := @FRecv[OldSize+1];
      FWSABuf.len := NeedToRead;
      FToRead := NeedToRead;
      FRead := 0; FWasRead := 0;
      Execute;
      Result := False;
    end else
      Result := True
  end
  else
  if (Found <> -1) then
  begin
    Tail := FWasRead - Found;
    ChannelImpl.Add2Cache(PChar(FRecv) + Found, Tail);
    Inc(FToRead, Tail); Dec(FWasRead, Tail);
    Dec(FTotalWasRead, Tail);
    SetLength(FRecv, Found);
    FEolFound := True;
    Result := True;
  end else
    Result := True;
end;

function TDnTcpLineRequest.IsCPUNeeded: Boolean;
begin
  Result := False;
end;

procedure TDnTcpLineRequest.ReExecute;
begin
  Execute;
end;

procedure TDnTcpLineRequest.CallHandler(Context: TDnThreadContext);
//var ChannelImpl: TDnTcpChannel;
begin
  try
    if FErrorCode = 0 then
    begin
      if FRead = 0 then
        FHandler.DoLineClose(Context, FChannel, FKey)
      else
        FHandler.DoLine(Context, FChannel, FKey, FRecv, FEolFound);
    end else
      FHandler.DoLineError(Context, FChannel, FKey, FErrorCode);
  finally
    //InterlockedDecrement(RequestsPending);
  end;
end;
//-----------------------------------------------------------------------------

constructor TDnTcpWriteRequest.Create(Channel: IDnChannel; Key: Pointer;
                                      Handler: IDnTcpWriteHandler; Buf: PChar;
                                      BufSize: Cardinal);
begin
  inherited Create(Channel, Key);

  FWSABuf.Len := BufSize;
  FWSABuf.Buf := Buf;
  FWritten := 0;
  FToWrite := BufSize;
  FFlags := 0;
  //FTotalSize := BufSize;
  FStartBuffer := @Buf;
  FHandler := Handler;
  FTempStorage := '';
end;

constructor TDnTcpWriteRequest.CreateString(Channel: IDnChannel; Key: Pointer;
                                      Handler: IDnTcpWriteHandler; Buf: String);
begin
  inherited Create(Channel, Key);

  FWSABuf.Len := Length(Buf);
  FWSABuf.Buf := PChar(Buf);
  FWritten := 0;
  FToWrite := Length(Buf);
  FFlags := 0;
  FTotalSize := Length(Buf);
  FStartBuffer := PChar(Buf);
  FHandler := Handler;
  FTempStorage := Buf;
end;

destructor TDnTcpWriteRequest.Destroy;
begin
  FHandler := Nil;
  inherited Destroy;
end;

function TDnTcpWriteRequest.RequestType: TDnIORequestType;
begin
  Result := rtWrite;
end;

procedure TDnTcpWriteRequest.SetTransferred(Transferred: Cardinal);
begin
  FWritten := Transferred;
  Dec(FWSABuf.len, FWritten);
  Inc(FWSABuf.buf, FWritten);
end;

procedure TDnTcpWriteRequest.Execute;
var ResCode: Integer;
    ChannelImpl: TDnTcpChannel;
begin
  inherited Execute;
  ChannelImpl := TDnTcpChannel.CheckImpl(FChannel);
  InterlockedIncrement(PendingRequests);
  ResCode := Winsock2.WSASend(ChannelImpl.SocketHandle, @FWSABuf , 1, FWritten, 0, @FContext, Nil);
  if ResCode = 0 then
  begin //WSASend completed immediately
    ;
  end else
  begin
    ResCode := WSAGetLastError;
    if (ResCode <> WSA_IO_PENDING)  then
      Self.PostError(ResCode, 0);
  end;
end;

function TDnTcpWriteRequest.IsComplete: Boolean;
begin
  inherited IsComplete;
  Result := (FWSABuf.len = 0) or (FErrorCode <> 0);
end;

function TDnTcpWriteRequest.IsCPUNeeded: Boolean;
begin
  Result := False;
end;

procedure TDnTcpWriteRequest.ReExecute;
begin
  Execute;
end;

procedure TDnTcpWriteRequest.CallHandler(Context: TDnThreadContext);
begin
  if FErrorCode = 0 then
    FHandler.DoWrite(Context, FChannel, FKey, FStartBuffer, FWritten)
  else
    FHandler.DoWriteError(Context, FChannel, FKey, FErrorCode);
  FTempStorage := '';
end;
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
end.
